==> atlassian-logging-2.15-sources.jar: com/atlassian/logging/log4j/appender/fluentd/LoggingEventQueue.java <==
package com.atlassian.logging.log4j.appender.fluentd;

import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.spi.LoggingEvent;

import javax.annotation.concurrent.ThreadSafe;


/**
 * A queue of events to send to fluentD.  Every batchPeriodMs, the contents will be sent to fluentd.
 *
 * We do not use ConcurrentLinkedQueue because we already have (and need) synchronized blocks around all the access
 * points to the queue. First, using ConcurrentLinkedQueue would be a redundant level of synchronization.  Furthermore,
 * when the queue is rotated in the sender for sendoff, we need to guarantee that no threads are still enqueuing onto it
 * so that we don't lose logs.  ConcurrentLinkedQueue is "wait-free", therefore it would return before the elements are
 * necessarily added to the queue causing us to lose that guarantee.
 */
@ThreadSafe
public class LoggingEventQueue
{
    private final long maxNumEvents;

    volatile LinkedList<LoggingEvent> loggingEventQueue = new LinkedList<>();

    public LoggingEventQueue(final long maxNumEvents)
    {
        this.maxNumEvents = maxNumEvents;
    }

    /**
     * Add a log event to the buffer.
     * Upon completion, if buffer limit is exceeded then old events are dropped until we have some space
     * @param loggingEvent
     */
    public synchronized void enqueue(final LoggingEvent loggingEvent)
    {
        loggingEventQueue.add(loggingEvent);

        while (loggingEventQueue.size() > maxNumEvents)
        {
            dequeue();
        }
    }

    /**
     * Pop up to <code>maximum</code> entries out of the queue
     * @param maximum maximum number of logging entries to retrieve. Pass a negative number to retrieve all
     * @return List of logging events
     */
    public synchronized List<LoggingEvent> retrieveLoggingEvents(final int maximum)
    {
        final List<LoggingEvent> events = new LinkedList<>();
        if (maximum < 0 || maximum >= loggingEventQueue.size())
        {
            // Just copy the queue over then reset it
            final List<LoggingEvent> currentQueue = loggingEventQueue;
            loggingEventQueue = new LinkedList<>();
            return currentQueue;
        }
        else
        {
            int remaining = maximum;
            while (remaining > 0)
            {
                events.add(dequeue());
                remaining--;
            }
        }
        return events;
    }

    /**
     * Remove and return the first log event
     * @return The first log event in the buffer
     */
    public synchronized LoggingEvent dequeue()
    {
        return loggingEventQueue.poll();
    }

    public int getSize()
    {
        return loggingEventQueue.size();
    }

    public boolean isFull()
    {
        return getSize() >= maxNumEvents;
    }
}

==> atlassian-logging-2.21-sources.jar: com/atlassian/logging/log4j/appender/fluentd/LoggingEventQueue.java <==
package com.atlassian.logging.log4j.appender.fluentd;

import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.spi.LoggingEvent;

import javax.annotation.concurrent.ThreadSafe;


/**
 * A queue of events to send to fluentD.  Every batchPeriodMs, the contents will be sent to fluentd.
 *
 * We do not use ConcurrentLinkedQueue because we already have (and need) synchronized blocks around all the access
 * points to the queue. First, using ConcurrentLinkedQueue would be a redundant level of synchronization.  Furthermore,
 * when the queue is rotated in the sender for sendoff, we need to guarantee that no threads are still enqueuing onto it
 * so that we don't lose logs.  ConcurrentLinkedQueue is "wait-free", therefore it would return before the elements are
 * necessarily added to the queue causing us to lose that guarantee.
 */
@ThreadSafe
public class LoggingEventQueue
{
    private final long maxNumEvents;

    volatile LinkedList<LoggingEvent> loggingEventQueue = new LinkedList<>();

    public LoggingEventQueue(final long maxNumEvents)
    {
        this.maxNumEvents = maxNumEvents;
    }

    /**
     * Add a log event to the buffer.
     * Upon completion, if buffer limit is exceeded then old events are dropped until we have some space
     * @param loggingEvent the event logging to enqueue.
     */
    public synchronized void enqueue(final LoggingEvent loggingEvent)
    {
        loggingEventQueue.add(loggingEvent);

        while (loggingEventQueue.size() > maxNumEvents)
        {
            dequeue();
        }
    }

    /**
     * Pop up to <code>maximum</code> entries out of the queue
     * @param maximum maximum number of logging entries to retrieve. Pass a negative number to retrieve all
     * @return List of logging events
     */
    public synchronized List<LoggingEvent> retrieveLoggingEvents(final int maximum)
    {
        final List<LoggingEvent> events = new LinkedList<>();
        if (maximum < 0 || maximum >= loggingEventQueue.size())
        {
            // Just copy the queue over then reset it
            final List<LoggingEvent> currentQueue = loggingEventQueue;
            loggingEventQueue = new LinkedList<>();
            return currentQueue;
        }
        else
        {
            int remaining = maximum;
            while (remaining > 0)
            {
                events.add(dequeue());
                remaining--;
            }
        }
        return events;
    }

    /**
     * Remove and return the first log event
     * @return The first log event in the buffer
     */
    public synchronized LoggingEvent dequeue()
    {
        return loggingEventQueue.poll();
    }

    public int getSize()
    {
        return loggingEventQueue.size();
    }

    public boolean isFull()
    {
        return getSize() >= maxNumEvents;
    }
}
